var FixedDataTable;

(function($, window, document) {		
/** 
 *  When using dataTables plug-in <a href='http://datatables.net'>http://datatables.net</a> <br/> 
 *  along with FixedColumns plug-in 
 *  <a href='http://www.datatables.net/extras/fixedcolumns/'>http://www.datatables.net/extras/fixedcolumns</a><br/>
 *  there are many inconsistencies between the normal behavior of dataTales
 *  and the mixed usage. <br/>
 *  For example the fnGetNodes will return rows but with columns that are not fixed.
 * <br/>
 *  FixedDataTable wraps the complex stuff between the mixed usage of dataTables
 *  and FixedColumns and provides useful methods, properties, events and call-backs.
 *  
 *  FixedDataTable expects a table with the following structure:
 *  <ol>
 *  <li>THEAD</li>
 *  <li>TBODY</li>
 *  </ol>
 *
 *  @class
 *  @constructor
 *  @param {FixedDataTable.Options} options
 *  @requires jQuery 1.4.2+
 *  @requires DataTables 1.7.6+
 *  @requires FixedColumns 2.0.0+
 * 
 *  @example
 * 		var options = new FixedDataTable.Options();
 *		options.id = 'resultTable';
 *		options.cssClass = 'resultTable';
 *		options.jQuery = false;				
 *		options.fixLeft = 4;
 *		options.sortDisable = [0],
 *		options.multipleSelect = false;
 *		options.onClick = function (clicked) {};
 *		options.checkboxSelection = true;
 *		options.checkboxClass = 'selectRow';
 *		options.checkboxSelectAllId = 'selectAll';
 *		options.onCheckboxClick = function ( checked, selectedRows ) {};
 *		options.columnTypes = {
 *			4 : 'uk_date',
 *			5 : 'uk_date'
 *		};
 *				
 *		table = new FixedDataTable( options );	
 */
FixedDataTable = function ( config ) {
	/* Check if dataTable is used properly*/
	if ( !(this instanceof FixedDataTable) )
	{
		alert( "DataTable warning: DataTable must be initialised with the 'new' keyword." );
		return;
	}
	
	if ( typeof config == 'undefined' )
	{
		config = {};
	}		
	
	/**
	 * A reference to the dataTable object
	 * <a href=http://www.datatables.net/>http://www.datatables.net</a>
	 * @type	Object
	 */
	this.dTable = null;
	
	/**
	 * Caches the result of that.dTable.fnSettings() when creating a new FixedDataTable
	 * @type Object
	 * @private 
	 */
	this._fnSettings = null;
	
	/**
	 * A reference to the dataTable object
	 * <a href='http://www.datatables.net/extras/fixedcolumns/'>http://www.datatables.net/extras/fixedcolumns</a>
	 * @type	FixedColumns
	 */
	this.fcTable = null;	

	/**
	 * The configuration options for the creation of the FixedDataTable object
	 * @type	FixedDataTable.Options
	 */
	this.elements = new FixedDataTable.Options();
	
	/**
	 * Contains a reference to the last clicked row
	 * @namespace
	 * @type Object
	 */
	this.clicked = {
			/**
			 * The row from the dataTable instance
			 * @type Object
			 */
			dtRow : null,
			
			/**
			 * The row from the fixedDataTable instance
			 * @type Object
			 */
			fcRow : null
	};	
		
	this._fnInit(config);	
}; 	

FixedDataTable.prototype = {
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * PUBLIC methods 
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
		
	/**
	 * It is preferable to call this method when you delete the table and no longer need it.
	 * It frees the resources atken by the FixedDataTable - mainly event handlers.
	 * 
	 */
	'fnDestroy' : function () {
		$('*').unbind('click.fixedDataTable');
	},
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Private methods (they are of course public in JS, but recommended as private)
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
			
	/**
	 * Performs a search for a row in a table by id of the row
	 * @param table - The table that contains the row we are interested in
	 * @param id - The id of the row
	 * @private
	 * @returns a reference to the row or null
	 */
	'_findRow' : function (table, id) {
		var found = $(table).find('tr#' + id);
		if ($(found).size() == 0) return null;
		return $(found)[0];
	},
	
	'_fnExtendDataTableSettings' : function () {
		var additionalSettings = {};
		var that = this;
		
		if ( this.elements.jQuery ) additionalSettings['bJQueryUI'] = true;			
		
		/* Disable sorting for sortDisable(this is an array of numbers) columns */			
		if ( this.elements.sortDisable && typeof this.elements.sortDisable  == 'object' ) {
			additionalSettings['aoColumnDefs'] = [{ "bSortable": false, "aTargets": this.elements.sortDisable }];	
		}	
		
		if ( that.elements.columnTypes ) {
			var columnTypes = new Array();
			$('#' + that.elements.id).find('tbody tr:first td').each( function (index, item) {				
				if ( that.elements.columnTypes[index] ) {
					columnTypes.push({ "sType": that.elements.columnTypes[index] });
				} else {
					columnTypes.push(null);
				}				
			});			
			additionalSettings['aoColumns'] = columnTypes;
		}
		
		$('#' + that.elements.id).find('thead td').each (function (index, item) {
			$(item).find('input.filter').attr('id', index);
		});
		
		var mergedSettings = $.extend(FixedDataTable.defaults.settings, additionalSettings);
		return mergedSettings;
	},
	
	'_fnInitFixedColumns' : function () {
		var that = this;
		var fixLeft = this.elements.fixLeft;			
		if ( fixLeft && isNumber(fixLeft) && fixLeft > 0) {
			this.fcTable = new FixedColumns( this.dTable, {
				'iLeftColumns': fixLeft							
			});
			
			$('.DTFC_Cloned tbody tr td *').live('change', function (event) {
				var fcRowIndex = $(this).parents('tbody:first').find('tr').index($(this).parents('tr:first'));				
				var dtRowIndex = that._fnSettings.aiDisplay[ that._fnSettings._iDisplayStart + fcRowIndex];
				var cell = $(this).parents('td:first');
				var columnIndex = $(this).parents('tr:first').find('td').index(cell);
				that._fnSettings.aoData[dtRowIndex]._anHidden[columnIndex] = $(cell).clone(true);								
			} );
			
		}
	},

	'_fnInitCheckboxSelectAll' : function () {
		var that = this;
		console.info("oki");
		$('#' + this.elements.checkboxSelectAllId).click( function (event) {
			var checked = $(this).attr('checked') ? true : false;									
			$(that.dTable.fnGetFilteredNodes()).each( function (inex, item) {				
				var fcRow = that._findRow(that.fcTable.dom.clone.left.body, item.id);
				if ( !fcRow ) {
					that.dTable.fnDisplayRow(item);
					fcRow = that._findRow(that.fcTable.dom.clone.left.body, item.id);
				}				

				$(fcRow).find('.' + that.elements.checkboxClass).attr('checked', checked);
				$(fcRow).find('.' + that.elements.checkboxClass).trigger('change');

				if ( that.elements.onCheckboxClick )					
					that.elements.onCheckboxClick.call(that, checked, {"dtRow": item, "fcRow": fcRow});
			} );
		} );
	},
	
	'_fnInitRowClicking' : function ( fixedTable ) {		
		var that = fixedTable;
		
		$('table.' + this.elements.cssClass).find('tbody tr').live('click.fixedDataTable', function (event) {			
			var table = $(this).parents('table:first');		
			
			if ( !that.elements.multipleSelect && that.clicked) {
				$(that.clicked.dtRow).removeClass("row_selected");
				if ( that.clicked.dtRow ) 
					$(that._findRow(that.fcTable.dom.clone.left.body, that.clicked.dtRow.id)).removeClass("row_selected");
			}
			
			if ( isCloning(table) ) {			
				that.clicked.dtRow = that._findRow(that.dTable, this.id);			
				that.clicked.fcRow = this;						
			} else {
				that.clicked.dtRow = this;			
				that.clicked.fcRow = that._findRow(that.fcTable.dom.clone.left.body, this.id);			
			}													
			
			if ( that.elements.multipleSelect && $(that.clicked.dtRow).hasClass("row_selected")) {
				$(that.clicked.dtRow).removeClass("row_selected");						
				$(that.clicked.fcRow).removeClass("row_selected");
			} else {
				$(that.clicked.dtRow).addClass("row_selected");						
				$(that.clicked.fcRow).addClass("row_selected");
			}			
			
			if ( $(event.target).hasClass(that.elements.checkboxClass) && that.elements.onCheckboxClick) {
				var checked = $(event.target).attr('checked') === 'checked' ? true : false;
				that.elements.onCheckboxClick.call(this, checked, that.clicked);								
			} else {
				if ( that.elements.onClick && typeof that.elements.onClick == "function") {				
					that.elements.onClick.call(that, that.clicked);
				}
			}					

		} );
			 
	},
	
	'_fnInitIndividualColumnFilters' : function () {
		var that = this;
		var fcTableHeader = that.fcTable.dom.clone.left.header;
		$(fcTableHeader).find("thead input.filter").each(function(index, item){
			var filterDefaultValue = "Search " + jQuery.trim($(fcTableHeader).find('.header:eq(' + index + ')').text()); 
			$(this).val(filterDefaultValue);
		});		
		
		$('.dataTables_scrollHead').find("thead input.filter").each(function(index, item){
			var filterDefaultValue = "Search " + jQuery.trim($('.dataTables_scrollHead').find('.header:eq(' + index + ')').text()); 
			$(this).val(filterDefaultValue);
		});	

		$("thead input.filter").live("focus click", function (event) {		
			if ( $(this).hasClass("search_init") )
			{
				$(this).removeClass("search_init");
				$(this).data('original', this.value);
				this.value = "";
			}
		} );
		
		$("thead input").live("blur", function (i) {
			if ( this.value == "" )
			{
				$(this).addClass("search_init");
				this.value = $(this).data('original');
			}
		} );
		
		$("thead input.filter").keyup( function () {		
			that.dTable.fnFilter( jQuery.trim(this.value), this.id );		
		} );
	},
	
	/**
	 * @private
	 * @param {FixedDataTable.Configurator} config configuration setting object
	 */
	'_fnInit' : function ( config ) {
		/* A reference to the instance. Than now can be uses in closures. */
		var that = this;
		
		$.extend(this.elements, config);
								
		if ( !this.elements.id || !this.elements.cssClass) {
			alert('DataTable id and cssClass parameters are obligatory!');
			return;
		}		
		
		if ( !!this.elements.checkboxSelection && !this.elements.checkboxClass) {
			alert('If you are going to use checkboxSelection for the table you should provide checkboxClass!');
			return;
		}	
		
		/* Add id attribute equal to the row index to each row */ 
		$('#' + this.elements.id + ' tbody tr').each( function (index, item) {					
			$(item).attr("id", index);
		} );																
				
		this.dTable = $('#' + this.elements.id).dataTable(this._fnExtendDataTableSettings());						
		this._fnSettings = this.dTable.fnSettings();
		
		this._fnInitFixedColumns();						
		
		this._fnInitIndividualColumnFilters();
				
		this._fnInitRowClicking( this );						
		
		if ( !!this.elements.checkboxSelectAllId ) {
			this._fnInitCheckboxSelectAll();
		}		
				
	}
};



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Static methods and properties(they are of course public in JS, but recommended as private)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/**
 * FixedDataTable default settings for initialisation
 *  @namespace
 *  @static
 */
FixedDataTable.defaults = {
	/**
	 * Options for creating dataTables
	 * @type Object
	 * @example
	 * 	settings : {		
	 *		'sScrollX': '100%',					
	 *		'bScrollCollapse': true,
	 *		'sPaginationType': 'full_numbers'
	 *	} 
	 */
	settings : {		
		'sScrollX': '100%',					
		'bScrollCollapse': true,
		'sPaginationType': 'full_numbers'
	}		
};

/**
 * Checks if a variable can represent a number
 * @param n - variable we want to check 
 * @returns {Boolean}
 */
function isNumber ( n ) {
	return !isNaN(parseFloat(n)) && isFinite(n);
};

/**
 * Check if table cloning - is actually FixedColumns table
 * @param table - The table onbject to check for
 */
function isCloning ( table ) {
	return $(table).hasClass('DTFC_Cloned');
};

})(jQuery, window, document);


(function($, window, document) {
	/**
	 * An Options object for the creation of FixedDataTable;
	 * @class
	 * @constructor
	 * @example
	 * 			var options = new FixedDataTable.Options();
	 * 			options.id = 'resultTable';
	 *			options.cssClass = 'resultTable';
	 *			options.jQuery = false;				
	 *			options.fixLeft = 4;
	 *			options.sortDisable = [0],
	 *			options.multipleSelect = false;
	 *			options.onClick = function (clicked) {};
	 *			options.checkboxSelection = true;
	 *			options.checkboxClass = 'selectRow';
	 *			options.checkboxSelectAllId = 'selectAll';
	 *			options.onCheckboxClick = function ( checked, selectedRows ) {};
	 */
	FixedDataTable.Options = function () {
		/* Check if FixedDataTable.Configurator is instantiated properly*/
		if ( !(this instanceof FixedDataTable.Options) )
		{
			alert( "FixedDataTable warning: Options must be initialised with the 'new' keyword." );
			return;
		}
		
		/**
		 * The html id attribute of the table element
		 * @type 	 String 
		 * @property
		 */
		this.id = "";
		
		/**
		 * The one of the values in the class attribute of the table
		 * @type	String
		 */
		this.cssClass = "";
		
		/**
		 * Using the jQuery theme style for the table(true) or the custom style (false)
		 * @type	Boolean
		 * @property 
		 */
		this.jQuery = false;
		
		/**
		 * Number of fixed left columns
		 * @type 	Integer
		 * @default 0
		 * @property
		 */
		this.fixLeft = 0;
		
		/**
		 * The columns that should not be sorted.
		 * Zero based array
		 * @type 	Array
		 * @default 0
		 * @example
		 * 		sortDisable: [0, 1, 2]
		 */
		this.sortDisable = null;
		
		/**
		 * TRUE each click on a row adds the class 'row_selected'<br/>
		 * FALSE each click on a row adds the class 'row_selected' and removes the class from all other
		 * @type	Boolean
		 * @default	false
		 */
		this.multipleSelect = false;
		
		/**
		 * Call-back function that is called when a row is clicked
		 * Accepts object parameter: Containing the row from fixedColumnsTable; the row from dataTables table
		 * @type	Function
		 * @example
		 * 		onClick: function (clicked) {
		 * 			var dataTableRow = clicked.dtRow;
		 * 			var fixedTableRow = clicked.fcRow;
		 * 		}
		 */
		this.onClick = null;
		
		/**
		 * TRUE If you have a checkbox as a selection element for the rows
		 * @default false
		 * @type 	Boolean
		 */
		this.checkboxSelection = false;
		
		/**
		 * The class attribute of the checkbox element
		 * @type	String
		 * @default ""
		 */
		this.checkboxClass = "";
		
		/**
		 * If you are using the checkbox element as a selection element for the rows
		 * It's worthy to have a select all checkbox. 
		 * This is the id of that element.
		 * @type 	String
		 * @default ""
		 */
		this.checkboxSelectAllId = "";
		
		/**
		 * A callback function accepting called when a checkbox element is clicked.
		 * Accepts 2 parameters:
		 * 		1. Booleand value indicating if the checkbox is checked
		 * 		2. Object parameter: Containing the row from fixedColumnsTable; the row from dataTables table
		 * @type 	Function
		 * @example
		 * 			onCheckboxClick: function ( checked, selectedRows ) {
		 * 				var dataTableRow = null, fixedTableRow = null;
		 * 				if ( checked === true) { 
		 * 					var dataTableRow = selectedRows.dtRow;
		 * 					var fixedTableRow = selectedRows.fcRow;
		 * 				}							
		 *			}
		 */
		this.onCheckboxClick = null;
		
		/**
		 * Columns types. Used for sorting and filtering.
		 * It's an object. Keys are the columns number (zero based).
		 * Supported types: <br/> 
		 * <ul>
		 * 	<li>uk_date - Date in the format: "dd/mm/yyyy". Example 1th of February 2010 - 01/02/2010</li>
		 * </ul>
		 * @type Object
		 * @example 
		 * 		columnTypes = {
		 * 			5 : "uk-date"
		 * 		}
		 */
		this.columnTypes = null;				
	};	
})(jQuery, window, document);



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Datatable.js plug-ins needed for FixedDataTable
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

$.fn.dataTableExt.oApi.fnGetFilteredNodes = function ( oSettings )
{
	var anRows = [];
	for ( var i=0, iLen=oSettings.aiDisplay.length ; i<iLen ; i++ )
	{
		var nRow = oSettings.aoData[ oSettings.aiDisplay[i] ].nTr;
		anRows.push( nRow );
	}
	return anRows;
};

$.fn.dataTableExt.oApi.fnDisplayRow = function ( oSettings, nRow )
{
	var iPos = -1;
	for( var i=0, iLen=oSettings.aiDisplay.length ; i<iLen ; i++ )
	{
		if( oSettings.aoData[ oSettings.aiDisplay[i] ].nTr == nRow )
		{
			iPos = i;
			break;
		}
	}
	
	if( iPos >= 0 )
	{
		oSettings._iDisplayStart = ( Math.floor(i / oSettings._iDisplayLength) ) * oSettings._iDisplayLength;
		this.oApi._fnCalculateEnd( oSettings );
	}
	
	this.oApi._fnDraw( oSettings );
};

jQuery.fn.dataTableExt.oSort['uk_date-asc']  = function(a,b) {
	var ukDatea = a.split('/');
	var ukDateb = b.split('/');
	
	var x = (ukDatea[2] + ukDatea[1] + ukDatea[0]) * 1;
	var y = (ukDateb[2] + ukDateb[1] + ukDateb[0]) * 1;
	
	return ((x < y) ? -1 : ((x > y) ?  1 : 0));
};

jQuery.fn.dataTableExt.oSort['uk_date-desc'] = function(a,b) {
	var ukDatea = a.split('/');
	var ukDateb = b.split('/');
	
	var x = (ukDatea[2] + ukDatea[1] + ukDatea[0]) * 1;
	var y = (ukDateb[2] + ukDateb[1] + ukDateb[0]) * 1;
	
	return ((x < y) ? 1 : ((x > y) ?  -1 : 0));
};